package tools.descartes.dql.connector.kieker.tests.util;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.fail;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;

import org.eclipse.xtext.junit4.InjectWith;
import org.eclipse.xtext.junit4.XtextRunner;
import org.eclipse.xtext.junit4.util.ParseHelper;
import org.junit.runner.RunWith;

import com.google.inject.Inject;

import tools.descartes.dql.lang.DescartesQLInjectorProvider;
import tools.descartes.dql.lang.descartesQL.DescartesQL;
import tools.descartes.dql.lang.descartesQL.ModelReferenceClause;
import tools.descartes.dql.lang.descartesQL.UsingClause;

@InjectWith(DescartesQLInjectorProvider.class)
@RunWith(XtextRunner.class)
public abstract class AbstractQueryTest implements QueryTestIface {

	protected static File createFileName(final String testCategory, final String testName) {
		final StringBuilder sb = new StringBuilder();
		File queryFile = null;

		// After HOURS (!) enjoying inconsistent Eclipse documentation about
		// platform URLs, APIs, etc. I just thought using a relative path might
		// work.... Aaaaaand it works. Finally.
		sb.append("../tools.descartes.dql.connector.kieker.tests");
		sb.append("/" + testCategory);
		sb.append("/" + testName);
		sb.append(EXTENSION);

		queryFile = new File(sb.toString());

		return queryFile;
	}

	private String queryRawText;

	private DescartesQL query;

	@Inject
	ParseHelper<DescartesQL> parser;

	public DescartesQL getQuery() {
		return query;
	}

	public String getQueryRawText() {
		return queryRawText;
	}

	@Override
	public void initialize(final File queryFile) throws Exception {
		if (!queryFile.isFile()) {
			fail("Query File not found: " + queryFile);
		}

		final FileReader queryFileReader = new FileReader(queryFile);
		final BufferedReader queryReader = new BufferedReader(queryFileReader);
		final StringBuilder builder = new StringBuilder();

		while (queryReader.ready()) {
			builder.append(queryReader.readLine()).append(" ");
		}

		queryReader.close();
		setQueryRawText(builder.toString());
		setQuery(parser.parse(getQueryRawText()));

		assertNotNull(getQuery());
	}

	public void setQuery(final DescartesQL query) {
		this.query = query;
	}

	public void setQueryRawText(final String queryRawText) {
		this.queryRawText = queryRawText;
	}

	public void testUsingClause(final UsingClause using) {
		
		assertNotNull(using);

		final ModelReferenceClause modelRef = using.getModelReference();
		assertNotNull(modelRef);

		assertEquals(MODEL_FAMILY, modelRef.getFamily().getFamily());		
	}
}
